{% import 'main.sv' as main with context %}
// This file was autogenerated by PeakRDL-uvm
package {{package_name}};
    `include "uvm_macros.svh"
    import uvm_pkg::*;
    {{ main.top()|indent}}
endpackage: {{package_name}}

